package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.LlmOrgConfigVO;
import com.qmth.ops.api.vo.OrgVO;
import com.qmth.ops.biz.domain.LlmModel;
import com.qmth.ops.biz.domain.LlmOrgConfig;
import com.qmth.ops.biz.domain.Org;
import com.qmth.ops.biz.query.LlmOrgConfigQuery;
import com.qmth.ops.biz.service.LlmModelService;
import com.qmth.ops.biz.service.LlmOrgConfigService;
import com.qmth.ops.biz.service.LlmSupplierService;
import com.qmth.ops.biz.service.OrgService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/llm/org_config")
public class LlmOrgConfigController {

    @Resource
    private OrgService orgService;

    @Resource
    private LlmSupplierService llmSupplierService;

    @Resource
    private LlmModelService llmModelService;

    @Resource
    private LlmOrgConfigService llmOrgConfigService;

    @PostMapping("/query")
    public IPage<LlmOrgConfigVO> query(LlmOrgConfigQuery query) {
        return llmOrgConfigService.query(query).convert(config -> {
            LlmModel model = llmModelService.getById(config.getModelId());
            Org org = orgService.getById(config.getOrgId());
            return new LlmOrgConfigVO(llmSupplierService.getById(model.getSupplierId()), model, new OrgVO(org), config);
        });
    }

    @PostMapping("/insert")
    public LlmOrgConfig insert(@RequestAttribute AdminSession adminSession, LlmOrgConfig config) {
        adminSession.hasPermission(Permission.LLM_ORG_CONFIG_INSERT);
        config.setLeftCount(config.getPermitCount());
        llmOrgConfigService.save(config);
        return config;
    }

    @PostMapping("/update/model")
    public LlmOrgConfig updateModel(@RequestAttribute AdminSession adminSession, LlmOrgConfig config) {
        adminSession.hasPermission(Permission.LLM_ORG_CONFIG_EDIT);
        llmOrgConfigService.updateModel(config);
        return llmOrgConfigService.findByOrgAndAppType(config.getOrgId(), config.getAppType());
    }

    @PostMapping("/update/permit_count")
    public LlmOrgConfig updatePermit(@RequestAttribute AdminSession adminSession, LlmOrgConfig config) {
        adminSession.hasPermission(Permission.LLM_ORG_CONFIG_EDIT);
        llmOrgConfigService.permit(config, config.getPermitCount());
        return llmOrgConfigService.findByOrgAndAppType(config.getOrgId(), config.getAppType());
    }

}
