package com.qmth.ops.biz.ai.client.aliyun;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.ChatRole;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.client.Credentials;
import com.qmth.ops.biz.ai.exception.ChatRequestError;

import java.io.IOException;

public class AliyunChatClient extends ChatApiClient {

    private static final String AUTH_HEADER_NAME = "Authorization";

    private static final String AUTH_HEADER_VALUE = "Bearer ";

    public AliyunChatClient(ChatApiConfig config) {
        super(config);
    }

    @Override
    protected Credentials gerCredentials() {
        return new Credentials(AUTH_HEADER_NAME, AUTH_HEADER_VALUE + getConfig().getSecret());
    }

    @Override
    protected Object buildRequest(ChatRequest request) {
        return new AliyunChatRequest(request, getConfig().getModel());
    }

    @Override
    protected ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        return mapper.readValue(data, AliyunChatResult.class).buildResult();
    }

    @Override
    protected ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        AliyunError error = null;
        if (data != null) {
            try {
                error = mapper.readValue(data, AliyunError.class);
            } catch (Exception e) {
            }
        }
        switch (statusCode) {
        case 400:
            throw new ChatRequestError(error != null ? error.getMessage() : "chat request error");
        case 429:
            throw new ReentrantException(error != null ? error.getMessage() : "chat model rate limit exceeded");
        default:
            throw new StatusException(error != null ? error.getMessage() : "chat model error");
        }
    }

    public static void main(String[] args) throws Exception {
        ChatApiConfig config = new ChatApiConfig();
        config.setSupplier("aliyun");
        config.setUrl("https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation");
        config.setSecret("sk-94599ccd34e240de8818310387386d91");
        config.setModel("qwen1.5-7b-chat");
        config.setQpm(0);
        AliyunChatClient client = new AliyunChatClient(config);
        ChatRequest request = new ChatRequest();
        request.addMessage(ChatRole.user, "who are you");
        System.out.println(new ObjectMapper().writeValueAsString(client.call(request)));
    }

}
