package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.ops.biz.dao.LlmPromptTemplateDao;
import com.qmth.ops.biz.domain.LlmPromptTemplate;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class LlmPromptTemplateService extends ServiceImpl<LlmPromptTemplateDao, LlmPromptTemplate> {

    private static final String CACHE_NAME = "llm_prompt_template";

    @Resource
    private LlmPromptTemplateDao llmPromptTemplateDao;

    @Cacheable(value = CACHE_NAME, key = "#modelId+'_'+#appType")
    public LlmPromptTemplate findByModelAndAppType(Long modelId, LlmAppType appType) {
        return llmPromptTemplateDao.selectOne(
                new LambdaQueryWrapper<LlmPromptTemplate>().eq(LlmPromptTemplate::getModelId, modelId)
                        .eq(LlmPromptTemplate::getAppType, appType));
    }

    public List<LlmPromptTemplate> findByModel(Long modelId) {
        return llmPromptTemplateDao
                .selectList(new LambdaQueryWrapper<LlmPromptTemplate>().eq(LlmPromptTemplate::getModelId, modelId));
    }

    @Transactional
    @CachePut(value = CACHE_NAME, key = "#template.modelId+'_'+template.appType", unless = "#template==null")
    public void update(LlmPromptTemplate template) {
        llmPromptTemplateDao
                .replace(template.getModelId(), template.getAppType(), template.getSystem(), template.getUser());
    }

}

